#include "baghira.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qdialog.h>
//#include <qsizegrip.h>
#include <qstatusbar.h>
#include <qgroupbox.h>

void LiquidStyle::polish( QPalette &pal ) {
	if ( inExitPolish )
		return ;

	// clear out all old colorized pixmaps
	int i;
	for ( i = 0; i < BITMAP_ITEMS; ++i ) {
		if ( pixmaps[ i ] ) {
			delete pixmaps[ i ];
			pixmaps[ i ] = NULL;
			}
		}
	// clear out all dicts
	btnDict.clear();
	btnShadowedDict.clear();
	bevelFillDict.clear();
	smallBevelFillDict.clear();
//	optionHandler->pixDict.clear();
	optionHandler->popupDict.clear();
	// reset brushes
	pagerHoverBrush = QBrush();
	pagerBrush = QBrush();

	/**
	 * Okay, here's the deal. We might need to set the application
	 * background brush with a pixmap stipple or a custom kicker color.
	 * It wouldn't be much of a problem but Qt calls this method for a lot
	 * of different palettes, not just the application's, so we really don't
	 * know what we are changing and if the QPalette parameter is actually
	 * the application's colors or something like a tooltip.
	 *
	 * This used to mean I'd check the timestamp of .qt/qtrc and reload the
	 * entire palette if it has changed. Now we just assume the QPalette
	 * parameter is the new application palette. Much more efficent :)
	 */

// 	if ( !isKicker && isPlain() )
// 		return ;

	bool newPalette = false;
	struct stat buffer;
	if ( stat( QFile::encodeName( QDir::homeDirPath() + "/.qt/qtrc" ), &buffer ) == 0 ) {
		unsigned int lastModTime = ( unsigned int ) buffer.st_mtime;
		if ( lastModTime > qtrcModificationTime ) {
			qtrcModificationTime = lastModTime;
			newPalette = true;
			}
		} else if ( !initialPaletteLoaded )  // Hack, should always have qtrc in KDE
		newPalette = true;
	initialPaletteLoaded = true;

	if ( !newPalette ) {
		pal = polishedPalette;
		return ;
		}
		
	pal.setColor( QColorGroup::Button, optionHandler->CustomButtonColor() );

	// only executed on first run or if qtrc has changed
	if ( isKicker ) {
		origPanelBrush = pal.brush( QPalette::Active,
		                            QColorGroup::Background );
		origPanelPalette = pal;
		QColor c( pal.active().button() );
		if ( optionHandler->usePanelCustomColor() &&
		        optionHandler->panelCustomColor().isValid() ){
			c = optionHandler->panelCustomColor();
		}

		pal.setColor( QColorGroup::Mid, c.dark( 110 ) );
		pal.setColor( QColorGroup::Dark, c.dark( 130 ) );
		pal.setColor( QColorGroup::Midlight, c.light( 110 ) );
		pal.setColor( QColorGroup::Light, c.light( 115 ) );
		if (optionHandler->BrushMe()){
			QBrush brush( c, gradBrush );
			pal.setBrush( QColorGroup::Background, brush );
		}
		else if ( optionHandler->usePanelStipple() ) {
			QPixmap stipple( 64, 64 );
			stipple.fill( c.rgb() );
			QPainter p;
			p.begin( &stipple );
			p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
			for ( i = 0; i < 63; i += 4 ) {
				p.drawLine( 0, i, 63, i );
				p.drawLine( 0, i+1, 63, i+1 );
				}
			p.end();
			QBrush brush( c, stipple );
			pal.setBrush( QColorGroup::Background, brush );
		} else
			pal.setBrush( QColorGroup::Background, c );
	}
	else if (optionHandler->BrushMe()){
			QColor c( pal.active().background() );
			QBrush brush( c, gradBrush );
			pal.setBrush( QColorGroup::Background, brush );
	}
	else if ( !isPlain() ) {
		origPanelBrush = pal.brush( QPalette::Active,QColorGroup::Button );
 		QColor c( pal.active().background() );
 			QPixmap stipple( 64, 64 );
			stipple.fill( c.rgb() );
			QPainter p;
			p.begin( &stipple );
			p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
			for ( i = 0; i < 63; i += 4 ) {
				p.drawLine( 0, i, 63, i );
				p.drawLine( 0, i+1, 63, i+1 );
			}
 			p.end();
			QBrush brush( c, stipple );
			pal.setBrush( QColorGroup::Background, brush );
	}
        
        // create nice bevel as highlight color
        QColor c(pal.active().highlight());
        QColor c2(c.light(140));
        QPixmap pix2 = QPixmap(48,48);
        QPainter coolPainter(&pix2);
        KPixmap pix = QPixmap(48,24);
        KPixmapEffect::gradient(pix,c,c2,KPixmapEffect::VerticalGradient,0);
        coolPainter.drawPixmap(0,0,pix);
        KPixmapEffect::gradient(pix,c2,c,KPixmapEffect::VerticalGradient,0);
        coolPainter.drawPixmap(0,24,pix);
        QBrush brush(c, pix2);
        pal.setBrush(QColorGroup::Highlight, brush);

	polishedPalette = pal;
	optionHandler->reset();
	}

void LiquidStyle::unPolish( QApplication *app ) {

	QPalette pal( app->palette() );
	inExitPolish = true;
// 	btnDict.setAutoDelete(true);btnDict.clear();
// 	btnShadowedDict.setAutoDelete(true);btnShadowedDict.clear();
// 	buttonGlowDict.setAutoDelete(true);buttonGlowDict.clear();
// 	comboDict.setAutoDelete(true);comboDict.clear();
// 	comboShadowedDict.setAutoDelete(true);comboShadowedDict.clear();
// 	tabDict.setAutoDelete(true);tabDict.clear();
// 	inverseTabDict.setAutoDelete(true);inverseTabDict.clear();
// 	rectBtnDict.setAutoDelete(true);rectBtnDict.clear();
// 	progress2Dict.setAutoDelete(true);progress2Dict.clear();
// 	bevelFillDict.setAutoDelete(true);bevelFillDict.clear();
// 	smallBevelFillDict.setAutoDelete(true);smallBevelFillDict.clear();
// 	optionHandler->popupDict.setAutoDelete(true);optionHandler->popupDict.clear();
	if ( isKicker ) {
		// reset kicker
		QColor c = origPanelBrush.color();
		pal.setBrush( QColorGroup::Background, c );
		pal.setBrush( QColorGroup::Mid, c.dark( 110 ) );
		pal.setBrush( QColorGroup::Dark, c.dark( 130 ) );
		pal.setBrush( QColorGroup::Midlight, c.light( 110 ) );
		pal.setBrush( QColorGroup::Light, c.light( 115 ) );
//		app->blockSignals( TRUE );
		app->setPalette( pal );
//		app->blockSignals( FALSE );
		} else if ( !isPlain() ) {
		// reset any stipples
		if ( pal.brush( QPalette::Active, QColorGroup::Background ).pixmap() )
			pal.setBrush( QColorGroup::Background, pal.active().background() );
		if ( pal.brush( QPalette::Active, QColorGroup::Button ).pixmap() )
			pal.setBrush( QColorGroup::Button, pal.active().button() );
//		app->blockSignals( TRUE );
		app->setPalette( pal );
//		app->blockSignals( FALSE );
		}

	/*if(isKicker || !isPlain())
	    pal.setBrush(QColorGroup::Highlight, pal.active().highlight());
	*/
	inExitPolish = false;
	}

void LiquidStyle::polish( QWidget *w ) {

	bool tmpBool = false;
        bool isViewport = qstrcmp( w->name(), "qt_viewport" ) == 0 || qstrcmp( w->name(), "qt_clipped_viewport" ) == 0;
	bool isViewportChild = w->parent() && ( ( qstrcmp( w->parent() ->name(), "qt_viewport" ) == 0 ) || ( qstrcmp( w->parent() ->name(), "qt_clipped_viewport" ) == 0 ) );
        
        if (w->inherits("KColorGrid")){
            // kiconedit doesn't check for the paintevent rect size, but tries to create a (possibly) NULL Pixmap (after palette change event e.g.)
            // so we catch it's paint event and destroy it if the size is NULL
            // remove this after kiconedit corrected its code
            w->installEventFilter( this );
            w->setBackgroundMode(QWidget::PaletteBase);
            goto kstpolish;
        }
        
        if ( w->inherits( "QPopupMenu" )){
		optionHandler->prepareMenus();
		w->installEventFilter( optionHandler);
		w->setBackgroundMode( NoBackground );
		goto kstpolish ;
	}

        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	//groupbox shadow stuff - don't try to catch by "w->inherits('QGroupBox')" or so. THIS HANDLES GROUPBOX CHILDREN
	if ( optionHandler->drawGroupBoxShadow() ){ // do we want nice sunken groupboxes with shadow?
		QWidget * w2;
		w2 = w->parentWidget(true);
		if ( w2 != 0 ) { // this check is VERY important !!! - leave it, and QT will fail to display anything (e.g. kdm won't start anymore)
			QGroupBox::QGroupBox * w3 = ( QGroupBox::QGroupBox* ) w2;
			// was my Daddy a Groupbox?
			bool iAmGroup = w2->inherits( "QGroupBox" ) && ( w3->frameShape() == QFrame::GroupBoxPanel ) && !(w3->isFlat()) && !( ( w->inherits( "QScrollView" ) ) || ( w->inherits( "QLineEdit" ) ) || ( w->inherits( "QComboBox" ) ));
			if ( iAmGroup ) { // if daddy is a groupbox...
				if ( isKicker ){ // kicker (and children) needs special handling again - otherwise the shadow pixmap won't be found and your GB will be completely black
					this->groupShadow = new QPixmap( 64, 64 );
					QColor c;
					( optionHandler->MenuColorButton() ) ? c = QColor( optionHandler->buttonColor() ) : c = QColor( optionHandler->bgColor() );
					this->groupShadow->fill( ( c.dark( 100 + optionHandler->ShadowDarkness() ) ).rgb() );
					if ( optionHandler->useBgStipple() ) {
						QPainter p;
						p.begin( this->groupShadow );
						p.setPen( c.dark( 100 + optionHandler->ShadowDarkness() + optionHandler->stippleContrast() ) );
						for ( int i = 0; i < 63; i += 4 ) {
							p.drawLine( 0, i, 63, i );
							p.drawLine( 0, i+1, 63, i+1 );
						}
					}
					w->setPaletteBackgroundPixmap ( *(this->groupShadow) );
				}
				else{
					w->setPaletteBackgroundPixmap ( *groupShadow ); // ... make me shadowed, too!
				}
			}
		}
	// cannot (!) return;, as the widget itself is unhadled...
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

// 	// first test, if we are toplevel...
	if (optionHandler->BrushMe() && w->isTopLevel() && !(w->inherits("QPopupMenu") || w->inherits("QMenuBar") || w->isDesktop() || w->inherits("QListBox") || w->inherits("QTipLabel") || w->inherits("AlphaWidget") || w->inherits("ScrollWidget") )){
		w->installEventFilter( this );
	}

        //if (testWidget) qWarning("testwidget stage %d",testInt++);
        
        if (w->inherits( "QPushButton" )){
		w->setBackgroundMode( QWidget::PaletteBackground);
                w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
//		return;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	if (w->inherits( "QComboBox" )){
		w->setBackgroundMode(NoBackground);
		w->installEventFilter( this );
                w->setBackgroundOrigin(QWidget::WindowOrigin);
//		return;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	if (w->inherits("TaskBar")){
		w->setBackgroundOrigin(QWidget::WidgetOrigin);
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	if ( isViewportChild && (w->inherits( "QRadioButton" ) || w->inherits( "QComboBox" ) || w->inherits( "QPushButton" ) )) {
	// shape radiobuttons on html pages...
		w->setAutoMask( true );
		w->setBackgroundMode( QWidget::NoBackground );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	if (w->inherits("KDockWidgetAbstractHeaderDrag")){
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	

	if (w->inherits("QSpinWidget")){
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	
	if ( !isPlain () && (w->inherits( "KonqIconViewWidget" ) || w->inherits( "khtml"))) {
	// if you want custom backgrounds in konqueror, you have to reset pixmap, since Konq animation can't handle it
	// original liquid did this on khtml, too (at speed loss) so well see, if we get errors from this...
		QPalette pal( w->palette() );
		pal.setBrush( QColorGroup::Background, pal.active().background() );
		w->setPalette( pal );
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
        if ( w->inherits( "KActiveLabel" )) {
            w->setBackgroundMode( QWidget::PaletteBackground );
            w->setBackgroundOrigin(QWidget::WindowOrigin);
        }
        if ((w->inherits( "QScrollView" ) || w->inherits("KateView") || w->inherits("TEWidget") || w->inherits("CHexViewWidget")) && !(w->isA("KOAgenda"))){
		w->setMouseTracking( true );
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
 	else if (( qstrcmp( qApp->argv() [ 0 ], "konqueror" ) == 0 ) && w->inherits("QStatusBar")) {
 		w->installEventFilter( this );
 		goto kstpolish;
 	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
        
        if (w->inherits("QSplitterHandle")) {
 		w->setBackgroundOrigin(QWidget::WindowOrigin);
 		goto kstpolish;
 	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
        if (w->inherits("QSlider")){
 		w->setBackgroundMode( QWidget::NoBackground );
                goto kstpolish;
 	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
// 	else if (w->isTopLevel() && (w->minimumSize() != w->maximumSize())){
// 		QSizeGrip *grip = new QSizeGrip(w);
// 		goto kstpolish;
// 	}
	
	
	if ( w->isA( "AppletHandleDrag" ) || w->inherits( "KMiniPagerButton" ) || w->inherits( "QSplitterHandle" ) || w->inherits( "TaskContainer" ) || w->inherits( "QLineEdit" ) ) {
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	

	if ( !isPlain() ) {
	if ( !isViewport && w->parent() && qstrcmp( w->parent() ->name(), "proxyview" ) == 0 ) {
			w->setBackgroundOrigin( QWidget::WindowOrigin );
			goto kstpolish ;
		}
                //if (testWidget) qWarning("testwidget stage %d",testInt++);
		

// 		if ( w->ownPalette() && !w->inherits( "QButton" ) && !w->inherits( "QComboBox" ) )
// 			goto kstpolish ;
	//if (testWidget) qWarning("testwidget stage %d",testInt++);

		if ( w->inherits( "PanelButtonBase" ) )
			goto kstpolish ;
	//if (testWidget) qWarning("testwidget stage %d",testInt++);	
			
		if ( !isKicker&& !isViewport && !isViewportChild && !w->testWFlags( WType_Popup ) && !w->inherits( "KDesktop" ) && !w->inherits( "PasswordDlg" ) ) {
			if ( w->backgroundMode() == QWidget::PaletteBackground || w->backgroundMode() == QWidget::PaletteButton ) {
				w->setBackgroundOrigin( QWidget::WindowOrigin );
				}
			}
		}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	if ( w->inherits( "QPushButton" ) || w->inherits( "QComboBox" )) goto kstpolish;
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	

	if ( w->inherits( "QMenuBar" ) ) {
            if (optionHandler->BrushMe() && w->isTopLevel()){
            QPalette pal(w->palette());
            QColor c( pal.active().background() );
            if ( optionHandler->usePanelCustomColor() && optionHandler->panelCustomColor().isValid() ){
                c = optionHandler->panelCustomColor();
            }
            if ( optionHandler->usePanelStipple() ) {
                QPixmap stipple( 64, 64 );
                stipple.fill( c.rgb() );
                QPainter p;
                p.begin( &stipple );
                p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
                for (int i = 0; i < 63; i += 4 ) {
                    p.drawLine( 0, i, 63, i );
                    p.drawLine( 0, i+1, 63, i+1 );
                }
                p.end();
                QBrush brush( c, stipple );
                pal.setBrush( QColorGroup::Background, brush );
            } else{
                pal.setBrush( QColorGroup::Background, c );
            }
            w->setPalette( pal );
            }
            w->setBackgroundMode( QWidget::PaletteBackground );
            w->setBackgroundOrigin(QWidget::WindowOrigin);
            w->installEventFilter( this );
            goto kstpolish ;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	if (w->inherits("QFrame")){
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		if (((QFrame *)w)->frameShape() >= QFrame::Box && ((QFrame *)w)->frameShape() <= QFrame::WinPanel){
//			qWarning("forcing down frameShape");
			((QFrame *)w)->setFrameShape(QFrame::StyledPanel);
		}
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	
	 if ( w->parentWidget() && ( ( w->inherits( "QListBox" ) && w->parentWidget()->inherits( "QComboBox" ) ) || w->inherits( "KCompletionBox" ) ) ) {
		QListBox* listbox = (QListBox*) w;
		listbox->setBackgroundMode( NoBackground );
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	
	if (w->inherits("QHeader")){
		w->setBackgroundMode( NoBackground );
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	
	if (w->inherits("QTabBar")){
//                qWarning("installing tab event");
		w->setMouseTracking( true );
		w->installEventFilter( this );
                w->setBackgroundOrigin(QWidget::WindowOrigin);
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);


	if (w->inherits("QProgressBar"))
	{
		w->setBackgroundMode( NoBackground );
		WidgetAnimator* progressAnimator = new WidgetAnimator(w, 2);
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
	
	if ( w->inherits("QToolBar") ) {
 		if (w->inherits("KToolBar")) ((QToolBar*)w)->boxLayout()->setSpacing( 0 );
		((QToolBar*)w)->setMargin( 4 );
                w->setBackgroundMode(PaletteBackground);
                w->setBackgroundOrigin(QWidget::WindowOrigin);
		goto kstpolish ;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

 	if (tmpBool = w->inherits( "QToolButton" ) ||  w->inherits( "QToolBarSeparator" ) ||  w->inherits( "KToolBarSeparator" ) ){
		if (tmpBool && w->parentWidget() && w->parentWidget()->inherits("QToolBar")){
			w->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
		}
 		w->setBackgroundMode( QWidget::PaletteBackground );
 		w->setBackgroundOrigin(QWidget::WindowOrigin);
 		w->installEventFilter( this );
 		goto kstpolish;
 	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);


	if ( w->inherits( "QLabel" )){
            if ( !isPlain() && !(w->inherits( "QTipLabel" ) || w->inherits( "KSystemTray" ))) {
                w->setBackgroundMode( QWidget::PaletteBackground );
		w->setBackgroundOrigin(QWidget::WindowOrigin);
            }
            else if ( w->inherits( "QTipLabel" ) ) {
			w->setPalette( tooltipPalette );
			w->installEventFilter( this );
                        goto kstpolish ;
	    }
            if (qstrcmp( w->name(), "kde toolbar widget" ) == 0 ) w->installEventFilter( this );
            goto kstpolish ;
        }
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
        
		

	if ( w->inherits( "QDockWindow" ) ) {
		w->setBackgroundMode( QWidget::PaletteBackground );
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
		goto kstpolish ;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	if (w->inherits( "QRadioButton" ) || w->inherits( "QCheckBox" )){
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
		goto kstpolish;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);


	if ( w->inherits( "QScrollBar" ) ) {
		w->setCursor(Qt::PointingHandCursor);
		w->installEventFilter( this );
		w->setBackgroundMode( QWidget::NoBackground );
		goto kstpolish ;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);

	if ( w->isTopLevel() && isKicker ) {
	// more kicker stuff
		// force extensions and child panels to use proper palette.
		if ( !w->inherits( "Panel" ) && !w->inherits( "KPanelApplet" ) ) {
			if (!origPanelPalette.active().brush( QColorGroup::Background ).pixmap() ) {
				if (optionHandler->BrushMe()) {
					QBrush brush( origPanelPalette.active().brush( QColorGroup::Background ) );
//					QPixmap brushed = optionHandler->TintBrush() ? *tintBrush(fetchImage( "brush-me" ), optionHandler->BrushTint()) : fetchImage("brush-me" );
					brush.setPixmap( gradBrush );
					origPanelPalette.setBrush( QColorGroup::Background, brush );
				} else if ( !isPlain()) {
				QBrush brush( origPanelPalette.active().brush( QColorGroup::Background ) );
				QColor c( brush.color() );
				QPixmap stipple( 64, 64 );
				stipple.fill( c.rgb() );
				QPainter p;
				p.begin( &stipple );
				p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
				for ( int i = 0; i < 63; i += 4 ) {
					p.drawLine( 0, i, 63, i );
					p.drawLine( 0, i+1, 63, i+1 );
				}
				p.end();
				brush.setPixmap( stipple );
				origPanelPalette.setBrush( QColorGroup::Background, brush );
			}
                        }
			w->setPalette( origPanelPalette );
		}
		goto kstpolish ;
	}
        //if (testWidget) qWarning("testwidget stage %d",testInt++);
kstpolish:	KStyle::polish( w ); // nothing of all above? do default kde stuff
	}



void LiquidStyle::unPolish( QWidget *w ) {

	bool isViewport;
	bool isViewportChild;

	if ( w->inherits("QToolBar") || qstrcmp( w->name(), "kde toolbar widget" ) == 0 ) {
		w->setBackgroundOrigin( QWidget::WidgetOrigin );
		w->unsetPalette();
		goto kstunpolish;
		}
	if ( w->inherits( "QPopupMenu" ) ) {
		w->unsetPalette();
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( optionHandler );
		goto kstunpolish;
		}
	if ( !isPlain () && w->inherits( "KonqIconViewWidget" ) ||
	        w->inherits( "KHTMLView" ) ) {
		w->unsetPalette();
		goto kstunpolish;
		}
	if (w->inherits("KateView") || w->inherits("TEWidget") || w->inherits("CHexViewWidget")){
//		w->setMouseTracking( false );
		w->removeEventFilter( this );
		goto kstunpolish;
	}

	if (w->isA( "QViewportWidget" ) || w->inherits( "QClipperWidget" ) )
		goto kstunpolish;

	if ( !isPlain() ) {
		if ( w->inherits( "KActiveLabel" ) || w->inherits( "QTipLabel" ) ) {
			w->unsetPalette();
			w->removeEventFilter( this );
			goto kstunpolish;
			}
		/*
		else if(w->inherits("KdmClock")){
		    ; // Check this!
		    return;
			}*/
		}
	else if ( w->inherits( "QMenuBar" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( this );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		goto kstunpolish;
		}else if ( w->inherits( "QDockWindow" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		w->removeEventFilter( this );
		goto kstunpolish;
		} else if ( w->inherits( "QComboBox" ) || w->inherits( "QPushButton" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( this );
		} else if ( w->inherits( "QRadioButton" ) ||
	            w->inherits( "QCheckBox" ) || w->inherits( "QScrollView" ) || w->isA( "AppletHandleDrag" ) ||
	            w->inherits( "KMiniPagerButton" ) || w->inherits( "QSplitterHandle" ) ||
	            w->inherits( "TaskContainer" ) || w->inherits( "QLineEdit" ) || w->inherits( "QListBox" ) || w->inherits( "QLabel" ) || w->inherits( "KCompletionBox" ) || w->inherits("KDockWidgetAbstractHeaderDrag") || w->inherits("QSpinWidget")) {
		} else if ( w->inherits( "QScrollBar" ) ) {
//		if (!w->isEnabled()) w->setEnabled(true);
		w->setBackgroundMode( QWidget::PaletteBackground );
		w->removeEventFilter( this );
		goto kstunpolish;
		} else if ( w->inherits( "QHeader" ) ) {
		w->setMouseTracking( false );
		w->removeEventFilter( this );
		}

	if ( w->inherits( "QToolButton" ) ) {
		w->removeEventFilter( this );
		w->unsetPalette();
		w->setBackgroundMode( QWidget::PaletteButton );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		goto kstunpolish;
		}

	if ( w->isTopLevel() && isKicker && !w->inherits( "Panel" ) &&
	        !w->inherits( "KPanelApplet" ) ) {
		w->unsetPalette();
		goto kstunpolish;
		}

	isViewport = qstrcmp( w->name(), "qt_viewport" ) == 0 ||
	                  qstrcmp( w->name(), "qt_clipped_viewport" ) == 0;
	isViewportChild = w->parent() &&
	                       ( ( qstrcmp( w->parent() ->name(), "qt_viewport" ) == 0 ) ||
	                         ( qstrcmp( w->parent() ->name(), "qt_clipped_viewport" ) == 0 ) );

	if ( isViewportChild ) {
		if ( w->inherits( "QRadioButton" ) || w->inherits( "QComboBox" ) || w->inherits( "QPushButton" ) ) {
			if ( isHTMLWidget( w ) ) {
				w->setAutoMask( false );
				w->setBackgroundMode( QWidget::PaletteBackground );
				goto kstunpolish;
				}
			}
		}

	if ( !isPlain() ) {
		if ( !isViewport && w->parent() && qstrcmp( w->parent() ->name(), "proxyview" ) == 0 ) {
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
			goto kstunpolish;
			}
		if ( w->ownPalette() && !w->inherits( "QButton" ) && !w->inherits( "QComboBox" ) )
			goto kstunpolish;
		if ( w->inherits( "PanelButtonBase" ) )
			goto kstunpolish;

		if ( !isViewport && !isViewportChild && !w->testWFlags( WType_Popup ) &&
		        !w->inherits( "KDesktop" ) && !w->inherits( "PasswordDlg" ) ) {

			if ( w->backgroundOrigin() == QWidget::WindowOrigin )
				w->setBackgroundOrigin( QWidget::WidgetOrigin );
			goto kstunpolish;
			}
		}
kstunpolish:	KStyle::unPolish( w );
	}
